# Copyright (c) 1994 Adam Karpierz
# SPDX-License-Identifier: Zlib

"""Public Python API of CRC package."""

from .__about__ import * ; del __about__  # type: ignore[name-defined]  # noqa

from ._crc import * ; del _crc  # type: ignore[name-defined]  # noqa
